==> PyBookReader-0.4.1.tar.gz: PyBookReader-0.4.1/pybookreader/scrolledtext.py <==
#!/usr/bin/env python
# -*- mode: python; coding: koi8-r -*-


import string
import gobject, gtk, pango
from gtk import TRUE, FALSE


##----------------------------------------------------------------------
##----------------------------------------------------------------------

class ScrolledText(gtk.TextView):

    text_buffer = None
    vadjustment = None
    hadjustment = None

    #font = 'Sans Bold 26'
    fg_color = '#000000'
    bg_color = '#b5bea5'

    text_wrap_mode = gtk.WRAP_WORD

    # <name> <value> <label> <place> <min> <max>
    text_properties = {
        'text_left_margin'        : [8, '_Left margin:',        0,   0, 40],
        'text_right_margin'       : [8, '_Right margin:',       1,   0, 40],
        'text_pixels_above_lines' : [0, 'Pixels _above lines:', 2, -10, 20],
        'text_pixels_below_lines' : [0, 'Pixels _below lines:', 3, -10, 20],
        'text_pixels_inside_wrap' : [0, 'Pixels _inside wrap:', 4, -10, 20]
        }

    scroll_properties = {
        'auto_scroll_increment'    : [1, '_Auto scroll increment:',     0, 1, 20],
        'key_scroll_increment'     : [4, '_Key scroll increment:' ,     1, 1, 20],
        'key_scroll_page_increment': [8, 'K_ey scroll page increment:', 2, 1, 40],
        }

    # <name> <value> <label> <place>
    scroll_type = {
        'smooth_step_scrolling': [1, 'Smooth _step scrolling', 0],
        'smooth_page_scrolling': [1, 'Smooth _page scrolling', 1]
        }

    auto_timer   = None
    auto_timeout = 50
    speed = 20


    key_scroll_page_value = 0
    key_scroll_flag = 0

    SCROLL_UP      = 1
    SCROLL_DOWN    = 2
    SCROLL_PG_UP   = 3
    SCROLL_PG_DOWN = 4

    keysyms_up      = (gtk.keysyms.KP_Up,
                       gtk.keysyms.Up)
    keysyms_down    = (gtk.keysyms.KP_Down,
                       gtk.keysyms.Down)
    keysyms_pg_up   = (gtk.keysyms.KP_Page_Up,
                       gtk.keysyms.Page_Up)
    keysyms_pg_down = (gtk.keysyms.KP_Page_Down,
                       gtk.keysyms.Page_Down,
                       gtk.keysyms.space)

    keysyms_pause_toggle = [gtk.keysyms.Insert,
                            gtk.keysyms.KP_Insert]
    keysyms_speed_up     = (gtk.keysyms.equal,
                            gtk.keysyms.KP_Add)
    keysyms_speed_down   = (gtk.keysyms.minus,
                            gtk.keysyms.KP_Subtract)


    pause = 1


    def __init__(self):


        gtk.TextView.__init__(self)


        gobject.signal_new('pause-toggled', ScrolledText,
                           gobject.SIGNAL_RUN_LAST,
                           gobject.TYPE_NONE,
                           (gobject.TYPE_INT,))

        gobject.signal_new('speed-changed', ScrolledText,
                           gobject.SIGNAL_RUN_LAST,
                           gobject.TYPE_NONE,
                           (gobject.TYPE_INT,))


        self.text_buffer = gtk.TextBuffer(None)


        self.set_buffer(self.text_buffer)

        self.set_editable(FALSE)
        self.set_cursor_visible(FALSE)

        self.vadjustment = gtk.Adjustment()
        self.hadjustment = gtk.Adjustment()

        self.set_scroll_adjustments(self.hadjustment, self.vadjustment)

        self.set_properties()


        self.set_property('height_request', 200)
        self.set_property('width_request', 300)


        self.set_wrap_mode(self.text_wrap_mode)


        self.connect('key-press-event', self.key_press_event_cb)
        self.connect('key-release-event', self.key_release_event_cb)
        self.connect('button_press_event',
                     self.text_view_button_press_event_cb)
        self.connect('button_release_event',
                     self.text_view_button_press_event_cb)

        # colors
        if self.fg_color:
            color = gtk.gdk.color_parse(self.fg_color)
            self.modify_text(gtk.STATE_NORMAL, color) # text fg

        if self.bg_color:
            color = gtk.gdk.color_parse(self.bg_color)
            self.modify_base(gtk.STATE_NORMAL, color) # text bg

##----------------------------------------------------------------------

    def pause_toggled(self, pause):

        self.pause = pause

        vadj = self.vadjustment
        if vadj.value >= vadj.upper - vadj.page_size:
            if self.auto_timer:
                gtk.timeout_remove(self.auto_timer)
                self.auto_timer = None

        elif pause:
            if self.auto_timer:
                gtk.timeout_remove(self.auto_timer)
                self.auto_timer = None

        else:
            if not self.auto_timer:
                vadj.set_value(vadj.value + 1)
                self.auto_timer = gtk.timeout_add(self.auto_timeout,
                                                  self.auto_scroll_win)

        self.emit('pause-toggled', pause)

##----------------------------------------------------------------------

    def set_properties(self):

        for i in self.text_properties:
            if i == 'text_left_margin':
                self.set_property('left_margin',
                                            self.text_properties[i][0])
                self.notify('left_margin')
            elif i == 'text_right_margin':
                self.set_property('right_margin',
                                            self.text_properties[i][0])
                self.notify('right_margin')
            elif i == 'text_pixels_above_lines':
                self.set_pixels_above_lines(self.text_properties[i][0])
            elif i == 'text_pixels_below_lines':
                self.set_pixels_below_lines(self.text_properties[i][0])
            elif i == 'text_pixels_inside_wrap':
                self.set_pixels_inside_wrap(self.text_properties[i][0])


##----------------------------------------------------------------------

    def set_position(self, position):

        vadj = self.vadjustment
        vadj.set_value(position * (vadj.upper-vadj.page_size))

##----------------------------------------------------------------------

    def get_position(self):

        vadj = self.vadjustment
        if vadj.upper != vadj.page_size:
            return vadj.value/(vadj.upper-vadj.page_size)
        else:
            return 1.0

##----------------------------------------------------------------------

    def text_view_button_press_event_cb(self, w, event):

        if event.type == gtk.gdk.BUTTON_PRESS:

            if event.button == 1:

                self.key_scroll_flag = self.SCROLL_DOWN
                gtk.idle_add(self.key_scroll_win)
                return TRUE

            elif event.button == 2:

                self.key_scroll_flag = self.SCROLL_UP
                gtk.idle_add(self.key_scroll_win)
                return TRUE


        elif event.type == gtk.gdk.BUTTON_RELEASE:

            self.key_scroll_flag = 0

        return FALSE

        #return TRUE # if TRUE text_view don't processed this event

##----------------------------------------------------------------------

    def key_press_event_cb(self, w, event):

        if event.type == gtk.gdk.KEY_PRESS:

            if event.keyval in self.keysyms_pause_toggle:

                self.pause_toggled(not self.pause)

            elif event.keyval in self.keysyms_speed_up:

                if self.speed < 1000:
                    self.speed += 1
                    self.speed_changed()
                    self.emit('speed-changed', self.speed)

            elif event.keyval in self.keysyms_speed_down:

                if self.speed > 0:
                    self.speed -= 1
                    self.speed_changed()
                    self.emit('speed-changed', self.speed)

            elif self.key_scroll_flag != 0:

                return TRUE

            else:
                vadj = self.vadjustment
                if event.keyval in self.keysyms_up:

                    if self.scroll_type['smooth_step_scrolling'][0]:
                        self.key_scroll_flag = self.SCROLL_UP
                        gtk.idle_add(self.key_scroll_win)
                    else:
                        decr = min(vadj.value, vadj.step_increment)
                        if decr > 0:
                            vadj.set_value(vadj.value - decr)

                elif event.keyval in self.keysyms_down:

                    if self.scroll_type['smooth_step_scrolling'][0]:
                        self.key_scroll_flag = self.SCROLL_DOWN
                        gtk.idle_add(self.key_scroll_win)
                    else:
                        incr = min(vadj.step_increment,
                                   vadj.upper-vadj.value-vadj.page_size)
                        if incr > 0:
                            vadj.set_value(vadj.value + incr)

                elif event.keyval in self.keysyms_pg_up:

                    if self.scroll_type['smooth_page_scrolling'][0]:
                        self.key_scroll_flag = self.SCROLL_PG_UP
                        self.key_scroll_page_value = vadj.page_size \
                            - vadj.step_increment
                        gtk.idle_add(self.key_scroll_win)
                    else:
                        decr = min(vadj.value, vadj.page_increment)
                        if decr > 0:
                            vadj.set_value(vadj.value - decr)

                elif event.keyval in self.keysyms_pg_down:

                    if self.scroll_type['smooth_page_scrolling'][0]:
                        self.key_scroll_flag = self.SCROLL_PG_DOWN
                        self.key_scroll_page_value = vadj.page_size \
                            - vadj.step_increment
                        gtk.idle_add(self.key_scroll_win)
                    else:
                        incr = min(vadj.page_increment,
                                   vadj.upper-vadj.value-vadj.page_size)
                        if incr > 0:
                            vadj.set_value(vadj.value + incr)



                if not self.pause:
                    gtk.timeout_remove(self.auto_timer)


            return TRUE
        else:
            return FALSE

##----------------------------------------------------------------------

    def key_release_event_cb(self, w, event):
        vadj = self.vadjustment

        if event.type == gtk.gdk.KEY_RELEASE:

            if event.keyval in self.keysyms_up:
                self.key_scroll_flag = 0
            elif event.keyval in self.keysyms_down:
                self.key_scroll_flag = 0
            elif event.keyval in self.keysyms_pg_up:
                pass
            elif event.keyval in self.keysyms_pg_down:
                pass


            return TRUE

        else:
            return FALSE

##----------------------------------------------------------------------

    def auto_scroll_win(self):
        vadj = self.vadjustment
        if vadj.value < vadj.upper - vadj.page_size:
            #gtk.gdk.window_process_all_updates() # NO!!!
            vadj.set_value(vadj.value \
                 + self.scroll_properties['auto_scroll_increment'][0])
            return TRUE
        else:
            #gtk.timeout_remove(self.auto_timer)
            self.auto_timer = None
            self.pause_toggled(1)
            return FALSE

##----------------------------------------------------------------------

    def key_scroll_win(self):
        vadj = self.vadjustment

        if self.key_scroll_flag == self.SCROLL_UP:

            if vadj.value > 0:
                vadj.set_value(vadj.value \
                     - self.scroll_properties['key_scroll_increment'][0])
                gtk.idle_add(self.key_scroll_win)
                return
            else:
                self.key_scroll_flag = 0

        elif self.key_scroll_flag == self.SCROLL_DOWN:

            if vadj.value < vadj.upper - vadj.page_size:
                vadj.set_value(vadj.value \
                     + self.scroll_properties['key_scroll_increment'][0])
                gtk.idle_add(self.key_scroll_win)
                return
            else:
                self.key_scroll_flag = 0
                if not self.pause:
                    self.pause_toggled(0)
        elif self.key_scroll_flag == self.SCROLL_PG_UP:

            if self.key_scroll_page_value <= 0 or vadj.value <= 0:
                self.key_scroll_page_value = 0
                self.key_scroll_flag = 0
            else:
                self.key_scroll_page_value = self.key_scroll_page_value \
                    - self.scroll_properties['key_scroll_page_increment'][0]
                vadj.set_value(vadj.value \
                    - self.scroll_properties['key_scroll_page_increment'][0])

                gtk.idle_add(self.key_scroll_win)
                return

        elif self.key_scroll_flag == self.SCROLL_PG_DOWN:

            if self.key_scroll_page_value <= 0 \
                   or vadj.value >= vadj.upper - vadj.page_size:

                self.key_scroll_page_value = 0
                self.key_scroll_flag = 0
            else:
                self.key_scroll_page_value = self.key_scroll_page_value \
                    - self.scroll_properties['key_scroll_page_increment'][0]
                vadj.set_value(vadj.value \
                    + self.scroll_properties['key_scroll_page_increment'][0])

                gtk.idle_add(self.key_scroll_win)
                return

        if not self.pause:
            self.auto_timer = gtk.timeout_add(self.auto_timeout,
                                              self.auto_scroll_win)


##----------------------------------------------------------------------

    def speed_changed(self):

        self.auto_timeout = 1000 / self.speed
        if self.auto_timer:
            gtk.timeout_remove(self.auto_timer)
            self.auto_timer = None
            self.auto_timer = gtk.timeout_add(self.auto_timeout,
                                              self.auto_scroll_win)


##----------------------------------------------------------------------
##----------------------------------------------------------------------

if(__name__=="__main__"):

    window = gtk.Window()
    st = ScrolledText()

    data = (('this is the test ' * 100) + '\n') * 100

    iter = st.text_buffer.get_iter_at_offset(0)
    st.scroll_type['smooth_step_scrolling'][0] = 0
    st.scroll_type['smooth_page_scrolling'][0] = 0

    st.text_buffer.insert(iter, data)

    window.add(st)
    window.connect('destroy', lambda w: gtk.main_quit())
    window.show_all()
    gtk.main()

==> PyBookReader-0.4.2.tar.gz: PyBookReader-0.4.2/pybookreader/scrolledtext.py <==
#!/usr/bin/env python
# -*- mode: python; coding: koi8-r -*-


import string
import gobject, gtk, pango
from gtk import TRUE, FALSE


##----------------------------------------------------------------------
##----------------------------------------------------------------------

class ScrolledText(gtk.TextView):

    text_buffer = None
    vadjustment = None
    hadjustment = None

    text_wrap_mode = gtk.WRAP_WORD

    # <name> <value> <label> <place> <min> <max>
    text_properties = {
        'text_left_margin'        : [8, '_Left margin:',        0,   0, 40],
        'text_right_margin'       : [8, '_Right margin:',       1,   0, 40],
        'text_pixels_above_lines' : [0, 'Pixels _above lines:', 2, -10, 20],
        'text_pixels_below_lines' : [0, 'Pixels _below lines:', 3, -10, 20],
        'text_pixels_inside_wrap' : [0, 'Pixels _inside wrap:', 4, -10, 20]
        }

    scroll_properties = {
        'auto_scroll_increment'    : [1, '_Auto scroll increment:',     0, 1, 20],
        'key_scroll_increment'     : [4, '_Key scroll increment:' ,     1, 1, 20],
        'key_scroll_page_increment': [8, 'K_ey scroll page increment:', 2, 1, 40],
        }

    # <name> <value> <label> <place>
    scroll_type = {
        'smooth_step_scrolling': [1, 'Smooth _step scrolling', 0],
        'smooth_page_scrolling': [1, 'Smooth _page scrolling', 1]
        }

    auto_timer   = None
    auto_timeout = 50
    speed = 20


    key_scroll_page_value = 0
    key_scroll_flag = 0

    SCROLL_UP      = 1
    SCROLL_DOWN    = 2
    SCROLL_PG_UP   = 3
    SCROLL_PG_DOWN = 4

    keysyms_up      = (gtk.keysyms.KP_Up,
                       gtk.keysyms.Up)
    keysyms_down    = (gtk.keysyms.KP_Down,
                       gtk.keysyms.Down)
    keysyms_pg_up   = (gtk.keysyms.KP_Page_Up,
                       gtk.keysyms.Page_Up)
    keysyms_pg_down = (gtk.keysyms.KP_Page_Down,
                       gtk.keysyms.Page_Down,
                       gtk.keysyms.space)

    keysyms_pause_toggle = [gtk.keysyms.Insert,
                            gtk.keysyms.KP_Insert]
    keysyms_speed_up     = (gtk.keysyms.equal,
                            gtk.keysyms.KP_Add)
    keysyms_speed_down   = (gtk.keysyms.minus,
                            gtk.keysyms.KP_Subtract)


    pause = 1


    def __init__(self):

        gtk.TextView.__init__(self)

        gobject.signal_new('pause-toggled', ScrolledText,
                           gobject.SIGNAL_RUN_LAST,
                           gobject.TYPE_NONE,
                           (gobject.TYPE_INT,))

        gobject.signal_new('speed-changed', ScrolledText,
                           gobject.SIGNAL_RUN_LAST,
                           gobject.TYPE_NONE,
                           (gobject.TYPE_INT,))

        #self.text_buffer = gtk.TextBuffer(None)
        #self.set_buffer(self.text_buffer)

        self.set_editable(FALSE)
        self.set_cursor_visible(FALSE)

        self.vadjustment = gtk.Adjustment()
        self.hadjustment = gtk.Adjustment()

        self.set_scroll_adjustments(self.hadjustment, self.vadjustment)

        self.set_properties()


        self.set_property('height_request', 200)
        self.set_property('width_request', 300)


        self.set_wrap_mode(self.text_wrap_mode)
        #self.set_justification(gtk.JUSTIFY_FILL) # don't support yet

        self.connect('key-press-event', self.key_press_event_cb)
        self.connect('key-release-event', self.key_release_event_cb)
        self.connect('button_press_event',
                     self.text_view_button_press_event_cb)
        self.connect('button_release_event',
                     self.text_view_button_press_event_cb)


##----------------------------------------------------------------------

    def pause_toggled(self, pause):

        self.pause = pause

        vadj = self.vadjustment
        if vadj.value >= vadj.upper - vadj.page_size:
            if self.auto_timer:
                gtk.timeout_remove(self.auto_timer)
                self.auto_timer = None

        elif pause:
            if self.auto_timer:
                gtk.timeout_remove(self.auto_timer)
                self.auto_timer = None

        else:
            if not self.auto_timer:
                vadj.set_value(vadj.value + 1)
                self.auto_timer = gtk.timeout_add(self.auto_timeout,
                                                  self.auto_scroll_win)

        self.emit('pause-toggled', pause)

##----------------------------------------------------------------------

    def set_properties(self):

        for i in self.text_properties:
            if i == 'text_left_margin':
                self.set_property('left_margin',
                                            self.text_properties[i][0])
                self.notify('left_margin')
            elif i == 'text_right_margin':
                self.set_property('right_margin',
                                            self.text_properties[i][0])
                self.notify('right_margin')
            elif i == 'text_pixels_above_lines':
                self.set_pixels_above_lines(self.text_properties[i][0])
            elif i == 'text_pixels_below_lines':
                self.set_pixels_below_lines(self.text_properties[i][0])
            elif i == 'text_pixels_inside_wrap':
                self.set_pixels_inside_wrap(self.text_properties[i][0])


##----------------------------------------------------------------------

    def set_position(self, position):

        vadj = self.vadjustment
        vadj.set_value(position * (vadj.upper-vadj.page_size))

##----------------------------------------------------------------------

    def get_position(self):

        vadj = self.vadjustment
        if vadj.upper != vadj.page_size:
            return vadj.value/(vadj.upper-vadj.page_size)
        else:
            return 1.0

##----------------------------------------------------------------------

    def text_view_button_press_event_cb(self, w, event):

        if event.type == gtk.gdk.BUTTON_PRESS:

            if event.button == 1:

                self.key_scroll_flag = self.SCROLL_DOWN
                gtk.idle_add(self.key_scroll_win)
                return TRUE

            elif event.button == 2:

                self.key_scroll_flag = self.SCROLL_UP
                gtk.idle_add(self.key_scroll_win)
                return TRUE

        elif event.type == gtk.gdk.BUTTON_RELEASE:

            self.key_scroll_flag = 0

        return FALSE

        #return TRUE # if TRUE text_view don't processed this event

##----------------------------------------------------------------------

    def key_press_event_cb(self, w, event):

        if event.type == gtk.gdk.KEY_PRESS:

            if event.keyval in self.keysyms_pause_toggle:

                self.pause_toggled(not self.pause)


            elif event.keyval in [gtk.keysyms.A, gtk.keysyms.a]:
                # FIXME: Control-A -- segfault ????
                print 'A'




            elif event.keyval in self.keysyms_speed_up:

                if self.speed < 1000:
                    self.speed += 1
                    self.speed_changed()
                    self.emit('speed-changed', self.speed)

            elif event.keyval in self.keysyms_speed_down:

                if self.speed > 0:
                    self.speed -= 1
                    self.speed_changed()
                    self.emit('speed-changed', self.speed)

            elif self.key_scroll_flag != 0:

                return TRUE

            else:
                vadj = self.vadjustment
                if event.keyval in self.keysyms_up:

                    if self.scroll_type['smooth_step_scrolling'][0]:
                        self.key_scroll_flag = self.SCROLL_UP
                        gtk.idle_add(self.key_scroll_win)
                    else:
                        decr = min(vadj.value, vadj.step_increment)
                        if decr > 0:
                            vadj.set_value(vadj.value - decr)

                elif event.keyval in self.keysyms_down:

                    if self.scroll_type['smooth_step_scrolling'][0]:
                        self.key_scroll_flag = self.SCROLL_DOWN
                        gtk.idle_add(self.key_scroll_win)
                    else:
                        incr = min(vadj.step_increment,
                                   vadj.upper-vadj.value-vadj.page_size)
                        if incr > 0:
                            vadj.set_value(vadj.value + incr)

                elif event.keyval in self.keysyms_pg_up:

                    if self.scroll_type['smooth_page_scrolling'][0]:
                        self.key_scroll_flag = self.SCROLL_PG_UP
                        self.key_scroll_page_value = vadj.page_size \
                            - vadj.step_increment
                        gtk.idle_add(self.key_scroll_win)
                    else:
                        decr = min(vadj.value, vadj.page_increment)
                        if decr > 0:
                            vadj.set_value(vadj.value - decr)

                elif event.keyval in self.keysyms_pg_down:

                    if self.scroll_type['smooth_page_scrolling'][0]:
                        self.key_scroll_flag = self.SCROLL_PG_DOWN
                        self.key_scroll_page_value = vadj.page_size \
                            - vadj.step_increment
                        gtk.idle_add(self.key_scroll_win)
                    else:
                        incr = min(vadj.page_increment,
                                   vadj.upper-vadj.value-vadj.page_size)
                        if incr > 0:
                            vadj.set_value(vadj.value + incr)


                else:
                    return FALSE

                if not self.pause:
                    gtk.timeout_remove(self.auto_timer)


            return TRUE
        else:
            return FALSE

##----------------------------------------------------------------------

    def key_release_event_cb(self, w, event):
        vadj = self.vadjustment

        if event.type == gtk.gdk.KEY_RELEASE:

            if event.keyval in self.keysyms_up:
                self.key_scroll_flag = 0
            elif event.keyval in self.keysyms_down:
                self.key_scroll_flag = 0
            elif event.keyval in self.keysyms_pg_up:
                pass
            elif event.keyval in self.keysyms_pg_down:
                pass


            return TRUE

        else:
            return FALSE

##----------------------------------------------------------------------

    def auto_scroll_win(self):
        vadj = self.vadjustment
        if vadj.value < vadj.upper - vadj.page_size:
            #gtk.gdk.window_process_all_updates() # NO!!!

            vadj.set_value(vadj.value \
                 + self.scroll_properties['auto_scroll_increment'][0])

            return TRUE
        else:
            #gtk.timeout_remove(self.auto_timer)
            self.auto_timer = None
            self.pause_toggled(1)
            return FALSE

##----------------------------------------------------------------------

    def key_scroll_win(self):
        vadj = self.vadjustment

        if self.key_scroll_flag == self.SCROLL_UP:

            if vadj.value > 0:
                vadj.set_value(vadj.value \
                     - self.scroll_properties['key_scroll_increment'][0])
                gtk.idle_add(self.key_scroll_win)
                return
            else:
                self.key_scroll_flag = 0

        elif self.key_scroll_flag == self.SCROLL_DOWN:

            if vadj.value < vadj.upper - vadj.page_size:
                vadj.set_value(vadj.value \
                     + self.scroll_properties['key_scroll_increment'][0])
                gtk.idle_add(self.key_scroll_win)
                return
            else:
                self.key_scroll_flag = 0
                if not self.pause:
                    self.pause_toggled(0)
        elif self.key_scroll_flag == self.SCROLL_PG_UP:

            if self.key_scroll_page_value <= 0 or vadj.value <= 0:
                self.key_scroll_page_value = 0
                self.key_scroll_flag = 0
            else:
                self.key_scroll_page_value = self.key_scroll_page_value \
                    - self.scroll_properties['key_scroll_page_increment'][0]
                vadj.set_value(vadj.value \
                    - self.scroll_properties['key_scroll_page_increment'][0])

                gtk.idle_add(self.key_scroll_win)
                return

        elif self.key_scroll_flag == self.SCROLL_PG_DOWN:

            if self.key_scroll_page_value <= 0 \
                   or vadj.value >= vadj.upper - vadj.page_size:

                self.key_scroll_page_value = 0
                self.key_scroll_flag = 0
            else:
                self.key_scroll_page_value = self.key_scroll_page_value \
                    - self.scroll_properties['key_scroll_page_increment'][0]
                vadj.set_value(vadj.value \
                    + self.scroll_properties['key_scroll_page_increment'][0])

                gtk.idle_add(self.key_scroll_win)
                return

        if not self.pause:
            self.auto_timer = gtk.timeout_add(self.auto_timeout,
                                              self.auto_scroll_win)


##----------------------------------------------------------------------

    def speed_changed(self):

        self.auto_timeout = 1000 / self.speed
        if self.auto_timer:
            gtk.timeout_remove(self.auto_timer)
            self.auto_timer = None
            self.auto_timer = gtk.timeout_add(self.auto_timeout,
                                              self.auto_scroll_win)


##----------------------------------------------------------------------
##----------------------------------------------------------------------

if(__name__=="__main__"):

    window = gtk.Window()
    st = ScrolledText()

    data = (('this is the test ' * 100) + '\n') * 100

    text_buffer = gtk.TextBuffer()
    st.set_buffer(text_buffer)

    iter = text_buffer.get_iter_at_offset(0)
    st.scroll_type['smooth_step_scrolling'][0] = 0
    st.scroll_type['smooth_page_scrolling'][0] = 0

    text_buffer.insert(iter, data)

    window.add(st)
    window.connect('destroy', lambda w: gtk.main_quit())
    window.show_all()
    gtk.main()
